@@ -67,7 +67,7 @@ warranty; not even for MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.\n\
 static void
 print_usage (FILE *stream, const char *program)
 {
-  fprintf (stream, _("Usage: %s [-m mindays][-M maxdays][-d lastday][-I inactive][-E expiredate][-W warndays] user\n"),
+  fprintf (stream, _("Usage: %s [-D binddn] [-m mindays][-M maxdays][-d lastday][-I inactive][-E expiredate][-W warndays] user\n"),
 	   program);
   fprintf (stream, _("       %s -l user\n"),
 	   program);
@@ -79,6 +79,9 @@ print_help (const char *program)
   print_usage (stdout, program);
   fprintf (stdout, _("%s - change user password expiry information\n\n"),
 	   program);
+  fputs (_("  -D binddn      Use dn \"binddn\" to bind to the LDAP directory\n"),
+         stdout);
+
   fputs (_("  -r service     Use nameservice 'service'\n"), stdout);
   fputs (_("  -q, --quiet    Don't be verbose\n"), stdout);
   fputs (_("      --help     Give this help list\n"), stdout);
@@ -496,28 +499,6 @@ main (int argc, char *argv[])
 
       caller_name = strdupa (pw->pw_name);
 
-#if 0
-      /* We change the shadow information for another user, get that
-	 data, too.  */
-      if (argc == 1)
-	{
-	  char *user = argv[0];
-
-	  while (getpwnam_r (user, &resultbuf, buffer, buflen, &pw) != 0
-		 && errno == ERANGE)
-	    {
-	      errno = 0;
-	      buflen += 256;
-	      buffer = alloca (buflen);
-	    }
-	  if (!pw)
-	    {
-	      fprintf (stderr, _("%s: Unknown user %s\n"), program, user);
-	      return E_UNKNOWN_USER;
-	    }
-	}
-#endif
-
       /* if we show/modify the data for another user, get the data from
 	 this one.  */
       if (argc == 1)
@@ -539,17 +520,30 @@ main (int argc, char *argv[])
     }
 
   /* Only root is allowed to change aging for local users. */
-  if (!l_flag && uid && pw_data->service == S_LOCAL)
+  if (!l_flag && uid && (pw_data->service == S_LOCAL ||
+			 (pw_data->service == S_LDAP && binddn == NULL)))
     {
       fprintf (stderr,
-	       _("Only root is allowed to change aging information.\n"));
+	       _("Only an administrator is allowed to change aging information.\n"));
       free_user_t (pw_data);
       return E_NOPERM;
     }
 
-  /* misuse l_flag: We don't need to extra ask for a password with "-l".  */
-  if (do_authentication (program, caller_name, pw_data, !l_flag) != 0)
-    return E_NOPERM;
+  if (binddn && pw_data->service == S_LDAP && !l_flag)
+    {
+      /* A user tries to change data stored in a LDAP database and
+	 knows the Manager dn, now we need the password from him.  */
+      char prompt[130+strlen (binddn)], *cp;
+
+      pw_data->binddn = strdup (binddn);
+      snprintf (prompt, sizeof (prompt), _("Enter LDAP Password:"));
+      cp = getpass (prompt);
+      pw_data->oldclearpwd = strdup (cp);
+    }
+  else
+    /* misuse l_flag: We don't need to extra ask for a password with "-l".  */
+    if (do_authentication (program, caller_name, pw_data, !l_flag) != 0)
+      return E_NOPERM;
 
   if (l_flag)
     {
@@ -577,7 +571,9 @@ main (int argc, char *argv[])
       return 0;
     }
 
-  if (uid != 0)
+  /* Caller must be root or he needs to know the binddn and password
+     for LDAP administrator.  */
+  if (uid != 0 && !(binddn && pw_data->service == S_LDAP))
     return E_USAGE;
 
   if (interactive)
@@ -685,6 +681,9 @@ main (int argc, char *argv[])
   else
     pw_data->sp_changed = TRUE;
 
+  if (binddn)
+    pw_data->binddn = strdup (binddn);
+
   if (do_setpwnam (pw_data) != 0)
     {
       fprintf (stderr,_( "Error while changing aging information.\n"));
