@@ -67,7 +67,7 @@ static int _set_ssl_default_options (ldap_session_t *);
 static int _set_ssl_options (ldap_session_t *);
 #endif
 
-static int _connect_anonymously (ldap_session_t * session);
+static int _connect_anonymously (ldap_session_t *session);
 #if defined(LDAP_API_FEATURE_X_OPENLDAP) && (LDAP_API_VERSION > 2000)
 #if LDAP_SET_REBIND_PROC_ARGS == 3
 static int _rebind_proc (LDAP * ld, LDAP_CONST char *url, ber_tag_t request,
@@ -88,7 +88,7 @@ static int _rebind_proc (LDAP * ld, char **whop, char **credp, int *methodp,
 #endif
 
 
-static int _connect_as_user (ldap_session_t * session,
+static int _connect_as_user (ldap_session_t *session,
 			     const char *password);
 static int _get_integer_value (LDAP * ld, LDAPMessage * e, const char *attr,
 			       int *ptr);
@@ -99,7 +99,7 @@ static int _get_string_value (LDAP * ld, LDAPMessage * e, const char *attr,
 static int _get_string_values (LDAP * ld, LDAPMessage * e, const char *attr,
 			       char ***ptr);
 static int _escape_string (const char *str, char *buf, size_t buflen);
-static int _reopen (ldap_session_t * session);
+static int _reopen (ldap_session_t *session);
 
 #ifdef HAVE_LIBPTHREAD
 #include <dlfcn.h>
@@ -770,8 +770,12 @@ open_ldap_session (ldap_session_t *session)
       int rc = ldap_initialize (&session->ld, session->conf->uri);
       if (rc != LDAP_SUCCESS)
 	{
-	  syslog (LOG_ERR, "ldap_initialize %s",
-		  ldap_err2string (rc));
+	  if (isatty (fileno (stderr)))
+	    fprintf (stderr, "ldap_initialize %s",
+		     ldap_err2string (rc));
+	  else
+	    syslog (LOG_ERR, "ldap_initialize %s",
+		    ldap_err2string (rc));
 	  return rc;
 	}
     }
@@ -795,8 +799,12 @@ open_ldap_session (ldap_session_t *session)
       int rc = ldap_set_option (session->ld, LDAP_OPT_X_TLS, &tls);
       if (rc != LDAP_SUCCESS)
 	{
-	  syslog (LOG_ERR, "ldap_set_option(LDAP_OPT_X_TLS) %s",
-		  ldap_err2string (rc));
+	  if (isatty (fileno (stderr)))
+	    fprintf (stderr, "ldap_set_option(LDAP_OPT_X_TLS) %s",
+		     ldap_err2string (rc));
+	  else
+	    syslog (LOG_ERR, "ldap_set_option(LDAP_OPT_X_TLS) %s",
+		    ldap_err2string (rc));
 	  return rc;
 	}
 
@@ -880,8 +888,12 @@ open_ldap_session (ldap_session_t *session)
 	  rc = ldap_start_tls_s (session->ld, NULL, NULL);
 	  if (rc != LDAP_SUCCESS)
 	    {
-	      syslog (LOG_ERR, "ldap_starttls_s: %s",
-		      ldap_err2string (rc));
+	      if (isatty (fileno (stderr)))
+		fprintf (stderr, "ldap_starttls_s: %s",
+			 ldap_err2string (rc));
+	      else
+		syslog (LOG_ERR, "ldap_starttls_s: %s",
+			ldap_err2string (rc));
 	      return rc;
 	    }
 	}
@@ -923,9 +935,13 @@ _set_ssl_default_options (ldap_session_t *session)
 			    session->conf->tls_cacertdir);
       if (rc != LDAP_SUCCESS)
 	{
-	  syslog (LOG_ERR,
-		  "ldap_set_option(LDAP_OPT_X_TLS_CACERTDIR): %s",
-		  ldap_err2string (rc));
+	  if (isatty (fileno (stderr)))
+	    fprintf (stderr, "ldap_set_option(LDAP_OPT_X_TLS_CACERTDIR): %s",
+		     ldap_err2string (rc));
+	  else
+	    syslog (LOG_ERR,
+		    "ldap_set_option(LDAP_OPT_X_TLS_CACERTDIR): %s",
+		    ldap_err2string (rc));
 	  return LDAP_OPERATIONS_ERROR;
 	}
     }
@@ -935,9 +951,13 @@ _set_ssl_default_options (ldap_session_t *session)
 			&session->conf->tls_checkpeer);
   if (rc != LDAP_SUCCESS)
     {
-      syslog (LOG_ERR,
-	      "ldap_set_option(LDAP_OPT_X_TLS_REQUIRE_CERT): %s",
-	      ldap_err2string (rc));
+      if (isatty (fileno (stderr)))
+	fprintf (stderr, "ldap_set_option(LDAP_OPT_X_TLS_REQUIRE_CERT): %s",
+		 ldap_err2string (rc));
+      else
+	syslog (LOG_ERR,
+		"ldap_set_option(LDAP_OPT_X_TLS_REQUIRE_CERT): %s",
+		ldap_err2string (rc));
       return LDAP_OPERATIONS_ERROR;
     }
 
@@ -948,9 +968,14 @@ _set_ssl_default_options (ldap_session_t *session)
 			    session->conf->tls_ciphers);
       if (rc != LDAP_SUCCESS)
 	{
-	  syslog (LOG_ERR,
-		  "ldap_set_option(LDAP_OPT_X_TLS_CIPHER_SUITE): %s",
-		  ldap_err2string (rc));
+	  if (isatty (fileno (stderr)))
+	    fprintf (stderr,
+		     "ldap_set_option(LDAP_OPT_X_TLS_CIPHER_SUITE): %s",
+		     ldap_err2string (rc));
+	  else
+	    syslog (LOG_ERR,
+		    "ldap_set_option(LDAP_OPT_X_TLS_CIPHER_SUITE): %s",
+		    ldap_err2string (rc));
 	  return LDAP_OPERATIONS_ERROR;
 	}
     }
@@ -961,9 +986,13 @@ _set_ssl_default_options (ldap_session_t *session)
 			    session->conf->tls_cert);
       if (rc != LDAP_SUCCESS)
 	{
-	  syslog (LOG_ERR,
-		  "ldap_set_option(LDAP_OPT_X_TLS_CERTFILE): %s",
-		  ldap_err2string (rc));
+	  if (isatty (fileno (stderr)))
+	    fprintf (stderr, "ldap_set_option(LDAP_OPT_X_TLS_CERTFILE): %s",
+		     ldap_err2string (rc));
+	  else
+	    syslog (LOG_ERR,
+		    "ldap_set_option(LDAP_OPT_X_TLS_CERTFILE): %s",
+		    ldap_err2string (rc));
 	  return LDAP_OPERATIONS_ERROR;
 	}
     }
@@ -974,9 +1003,14 @@ _set_ssl_default_options (ldap_session_t *session)
 			    session->conf->tls_key);
       if (rc != LDAP_SUCCESS)
 	{
-	  syslog (LOG_ERR,
-		  "ldap_set_option(LDAP_OPT_X_TLS_KEYFILE): %s",
-		  ldap_err2string (rc));
+	  if (isatty (fileno (stderr)))
+	    fprintf (stderr,
+		     "ldap_set_option(LDAP_OPT_X_TLS_KEYFILE): %s",
+		     ldap_err2string (rc));
+	  else
+	    syslog (LOG_ERR,
+		    "ldap_set_option(LDAP_OPT_X_TLS_KEYFILE): %s",
+		    ldap_err2string (rc));
 	  return LDAP_OPERATIONS_ERROR;
 	}
     }
@@ -993,7 +1027,7 @@ _set_ssl_options (ldap_session_t *session __UNUSED__)
 #endif
 
 static int
-_connect_anonymously (ldap_session_t * session)
+_connect_anonymously (ldap_session_t *session)
 {
   int rc;
   int msgid;
@@ -1053,7 +1087,7 @@ _connect_anonymously (ldap_session_t * session)
   if (rc != LDAP_SUCCESS)
     {
       if (isatty (fileno (stderr)))
-	fprintf (stderr, "error trying t bind (%s)\n",
+	fprintf (stderr, "error trying to bind (%s)\n",
 		 ldap_err2string (rc));
       else
 	syslog (LOG_ERR, "error trying to bind (%s)",
@@ -1167,7 +1201,7 @@ _rebind_proc (LDAP * ld, char **whop, char **credp, int *methodp, int freeit)
 
 
 static int
-_connect_as_user (ldap_session_t * session, const char *password)
+_connect_as_user (ldap_session_t *session, const char *password)
 {
   int rc, msgid;
 #if defined(HAVE_LDAP_PARSE_RESULT) && defined(HAVE_LDAP_CONTROLS_FREE)
@@ -1607,7 +1641,7 @@ _reopen (ldap_session_t *session)
 }
 
 int
-ldap_authentication (ldap_session_t * session,
+ldap_authentication (ldap_session_t *session,
 		    const char *user, const char *password)
 {
   int rc = 0;
@@ -1633,82 +1667,74 @@ int
 ldap_update_shell (ldap_session_t *session, const char *user,
 		   const char *old_password, const char *new_shell)
 {
-  LDAPMod *mods[2], mod;
-  char *strvals[2];
-  int rc = 0;
-
-  if (session->info == NULL)
-    {
-      rc = get_ldapuser_info (session, user);
-      if (rc != 0)
-	return rc;
-    }
-
-  if (!session->conf->rootbinddn || geteuid () != 0)
-    {
-      /* We're not root or don't have a rootbinddn so
-	 let's try binding as the user.  */
-      rc = _reopen (session);
-      if (rc != 0)
-	return rc;
-
-      rc = _connect_as_user (session, old_password);
-      if (rc != 0)
-	return rc;
-    }
-
-  /* update loginShell */
-  strvals[0] = strdupa (new_shell);
-  strvals[1] = NULL;
-
-  mod.mod_values = strvals;
-  mod.mod_type = (char *) "loginShell";
-  mod.mod_op = LDAP_MOD_REPLACE;
-
-  mods[0] = &mod;
-  mods[1] = NULL;
-
-  rc = ldap_modify_s (session->ld, session->info->userdn, mods);
-  _reopen (session);
-  _connect_anonymously (session);
-
-  return rc;
+  return ldap_update_field (session, user, NULL, old_password,
+			    "loginShell", new_shell);
 }
 
 int
 ldap_update_gecos (ldap_session_t *session, const char *user,
 		   const char *old_password, const char *new_gecos)
+{
+  return ldap_update_field (session, user, NULL, old_password,
+			    "gecos", new_gecos);
+}
+
+int
+ldap_update_field (ldap_session_t *session, const char *user,
+		   const char *binddn, const char *bindpw,
+		   const char *field, const char *new_value)
 {
   LDAPMod *mods[2], mod;
   char *strvals[2];
   int rc = 0;
+  char *cp_binddn = NULL;
+  char *cp_bindpw = NULL;
 
-  if (session->info == NULL)
+  if (binddn != NULL)
+    {
+      cp_binddn = session->conf->binddn;
+      cp_bindpw = session->conf->bindpw;
+      session->conf->binddn = strdup(binddn);
+      session->conf->bindpw = strdup(bindpw);
+    }
+
+  if (session->info == NULL && user != NULL)
     {
       rc = get_ldapuser_info (session, user);
       if (rc != 0)
-	return rc;
+	{
+	  if (binddn != NULL)
+	    {
+	      free (session->conf->binddn);
+	      free (session->conf->bindpw);
+	      session->conf->binddn = cp_binddn;
+	      session->conf->bindpw = cp_bindpw;
+	    }
+	  return rc;
+	}
     }
 
-  if (!session->conf->rootbinddn || geteuid () != 0)
+  if ((!session->conf->rootbinddn || (geteuid () != 0))
+      && (binddn == NULL))
     {
       /* We're not root or don't have a rootbinddn so
-	 let's try binding as the user.  */
+	 let's try binding as the user. Exept we have a
+	 binddn for this case.  */
       rc = _reopen (session);
       if (rc != 0)
 	return rc;
 
-      rc = _connect_as_user (session, old_password);
+      rc = _connect_as_user (session, bindpw);
       if (rc != 0)
 	return rc;
     }
 
-  /* update gecos */
-  strvals[0] = strdupa (new_gecos);
+  /* update field */
+  strvals[0] = strdupa (new_value);
   strvals[1] = NULL;
 
   mod.mod_values = strvals;
-  mod.mod_type = (char *) "gecos";
+  mod.mod_type = strdupa (field);
   mod.mod_op = LDAP_MOD_REPLACE;
 
   mods[0] = &mod;
@@ -1718,6 +1744,14 @@ ldap_update_gecos (ldap_session_t *session, const char *user,
   _reopen (session);
   _connect_anonymously (session);
 
+  if (binddn != NULL)
+    {
+      free (session->conf->binddn);
+      free (session->conf->bindpw);
+      session->conf->binddn = cp_binddn;
+      session->conf->bindpw = cp_bindpw;
+    }
+
   return rc;
 }
 
