@@ -132,6 +132,7 @@ display_pw (const struct passwd *pw)
  *      -f      execute chfn command to interpret flags
  *      -s      execute chsh command to interpret flags
  *      -k      change password only if expired
+ *      -D binddn Use binddn for LDAP binding
  *
  *      (*) requires administrator permission to execute.
  *
@@ -144,7 +145,7 @@ static void
 print_usage (FILE *stream, const char *program)
 {
   fprintf (stream, _("Usage: %s [-f|-g|-s|-k[-q]] [name]\n"), program);
-  fprintf (stream, _("       %s [-n min] [-x max] [-w warn] [-i inact] user\n"), program);
+  fprintf (stream, _("       %s [-D binddn] [-n min] [-x max] [-w warn] [-i inact] user\n"), program);
   fprintf (stream, _("       %s {-l|-u|-d|-S[-a]|-e|-h} user\n"), program);
 }
 
@@ -171,6 +172,8 @@ print_help (const char *program)
   fputs (_("  -x max         Set maximum field for \"user\"\n"), stdout);
   fputs (_("  -w warn        Set warn field for \"user\"\n"), stdout);
   fputs (_("  -r service     Use nameservice 'service'\n"), stdout);
+  fputs (_("  -D binddn      Use dn \"binddn\" to bind to the LDAP directory\n"),
+	 stdout);
   fputs (_("  --help         Give this help list\n"), stdout);
   fputs (_("  --usage        Give a short usage message\n"), stdout);
   fputs (_("  --version      Print program version\n"), stdout);
@@ -186,6 +189,7 @@ passwd_main (const char *program, int argc, char **argv)
   user_t *pw_data = NULL;
   char *caller_name = NULL;
   char *use_service = NULL;
+  char *binddn = NULL;
   int k_flag = 0, a_flag = 0, d_flag = 0, e_flag = 0, h_flag = 0,
     i_flag = 0, l_flag = 0, n_flag = 0, u_flag = 0, x_flag = 0,
     S_flag = 0, w_flag = 0;
@@ -198,6 +202,7 @@ passwd_main (const char *program, int argc, char **argv)
       int option_index = 0;
       static struct option long_options[] =
 	{
+	  {"binddn", required_argument, NULL, 'D'},
 	  {"service", required_argument, NULL, 'r'},
 	  {"version", no_argument, NULL, '\255'},
 	  {"usage", no_argument, NULL, '\254'},
@@ -205,7 +210,7 @@ passwd_main (const char *program, int argc, char **argv)
 	  {NULL, 0, NULL, '\0'}
 	};
 
-      c = getopt_long (argc, argv, "adhlqr:uSekn:x:i:w:", long_options,
+      c = getopt_long (argc, argv, "adD:hlqr:uSekn:x:i:w:", long_options,
                        &option_index);
       if (c == EOF)
         break;
@@ -218,6 +223,9 @@ passwd_main (const char *program, int argc, char **argv)
           d_flag = 1;
           admin_only = 1;
           break;
+	case 'D':
+	  binddn = optarg;
+	  break;
         case 'e':
           e_flag = 1;
           admin_only = 1;
@@ -492,6 +500,15 @@ passwd_main (const char *program, int argc, char **argv)
       return E_SUCCESS;
     }
 
+  if (binddn)
+    {
+      char prompt[130+strlen (binddn)], *cp;
+
+      pw_data->binddn = strdup (binddn);
+      snprintf (prompt, sizeof (prompt), _("Enter LDAP Password:"));
+      cp = getpass (prompt);
+      pw_data->oldclearpwd = strdup (cp);
+    }
 
   if (d_flag)
     pw_data->newpassword = strdup ("");
@@ -576,7 +593,11 @@ passwd_main (const char *program, int argc, char **argv)
 
   if (do_setpwnam (pw_data) != 0)
     {
-      fprintf (stderr, "Error while changing login shell\n");
+      if (pw_data->sp_changed)
+	fprintf (stderr,
+		 _("Error while changing password expiry information.\n"));
+      else
+	fprintf (stderr, _("Error while changing password.\n"));
       free_user_t (pw_data);
       return E_FAILURE;
     }
@@ -584,7 +605,12 @@ passwd_main (const char *program, int argc, char **argv)
     {
       nscd_flush_cache ("passwd");
       if (!silent)
-        printf (_("Password changed.\n"));
+	{
+	  if (pw_data->sp_changed)
+	    printf (_("Password expiry information changed.\n"));
+	  else
+	    printf (_("Password changed.\n"));
+	}
     }
 
   free_user_t (pw_data);
