@@ -212,7 +212,7 @@ server_init (int port, int ipv4, int ipv6)
       memset (&sock_addr, '\0', sizeof (sock_addr));
       sock_addr.sin6_family = AF_INET6;
       sock_addr.sin6_addr = in6addr_any;
-      sock_addr.sin6_port = htons (port);
+      sock_addr.sin6_port = port;
 
       if (bind (sock6, (struct sockaddr *) &sock_addr, sizeof (sock_addr)) < 0)
 	{
@@ -244,7 +244,7 @@ server_init (int port, int ipv4, int ipv6)
       memset (&sock_addr, '\0', sizeof (sock_addr));
       sock_addr.sin_family = AF_INET;
       sock_addr.sin_addr.s_addr = INADDR_ANY;
-      sock_addr.sin_port = htons (port);
+      sock_addr.sin_port = port;
 
       if (bind (sock4, (struct sockaddr *) &sock_addr, sizeof (sock_addr)) < 0)
 	{
@@ -1048,7 +1048,7 @@ main (int argc, char **argv)
 	  go_background = 0;
 	  break;
 	case 'p':
-	  port = atol (optarg);
+	  port = htons (atol (optarg));
 	  break;
         case '\253':
           print_help (program);
@@ -1139,12 +1139,14 @@ main (int argc, char **argv)
 
   if (SSL_CTX_use_certificate_file (ctx, certificate, SSL_FILETYPE_PEM) <= 0)
     {
-      dbg_log (ERR_error_string (ERR_get_error (), NULL));
+      dbg_log ("Loading certificate (%s): %s", certificate,
+	       ERR_error_string (ERR_get_error (), NULL));
       return E_SSL_FAILURE;
     }
   if (SSL_CTX_use_PrivateKey_file (ctx, privatekey, SSL_FILETYPE_PEM) <= 0)
     {
-      dbg_log (ERR_error_string (ERR_get_error (), NULL));
+      dbg_log ("Loading privatekey (%s): %s", privatekey,
+	       ERR_error_string (ERR_get_error (), NULL));
       return E_SSL_FAILURE;
     }
 
@@ -1164,7 +1166,7 @@ main (int argc, char **argv)
       if (serv)
 	port = serv->s_port;
       else
-	port = RPASSWDD_PORT;
+	port = htons (RPASSWDD_PORT);
     }
 
   /* Init databases.  */
